@@ -3,10 +3,10 @@ from __future__ import annotations
 import io
 import json
 import logging
-from dataclasses import dataclass, field
+from dataclasses import dataclass, field, replace
 from enum import Enum
 from pathlib import Path
-from typing import Dict, Set, Tuple, List, Optional
+from typing import Dict, Set, List, Optional
 
 import typer
 
@@ -20,13 +20,8 @@ class CoverageType(str, Enum):
     TEST = 'test'
 
 
-class EntityWithCoverage:
-    def coverage(self, cov_type: CoverageType) -> Tuple[Set[str], Set[str]]:
-        raise NotImplementedError()
-
-
 @dataclass
-class Column(EntityWithCoverage):
+class Column:
     """Dataclass containing the information about the docs and tests of a database column."""
 
     name: str
@@ -36,21 +31,9 @@ class Column(EntityWithCoverage):
     @staticmethod
     def from_node(node) -> Column:
         return Column(
-            node['name']
+            node['name'].lower()
         )
 
-    def coverage(self, cov_type: CoverageType) -> Tuple[Set[str], Set[str]]:
-        if cov_type == CoverageType.DOC:
-            covered = self.doc
-        elif cov_type == CoverageType.TEST:
-            covered = self.test
-        else:
-            raise ValueError(f"Unsupported cov_type {cov_type}")
-
-        covered = {self.name} if covered else set()
-        total = {self.name}
-        return covered, total
-
     @staticmethod
     def is_valid_doc(doc):
         return doc is not None and doc != ''
@@ -61,7 +44,7 @@ class Column(EntityWithCoverage):
 
 
 @dataclass
-class Table(EntityWithCoverage):
+class Table:
     """Dataclass containing the information about a database table and its columns."""
 
     name: str
@@ -69,55 +52,71 @@ class Table(EntityWithCoverage):
 
     @staticmethod
     def from_node(node) -> Table:
+        columns = [Column.from_node(col) for col in node['columns'].values()]
         return Table(
-            node['metadata']['name'],
-            {col['name']: Column.from_node(col) for col in node['columns'].values()}
+            f"{node['metadata']['schema']}.{node['metadata']['name']}".lower(),
+            {col.name: col for col in columns}
         )
 
     def get_column(self, column_name):
         return self.columns.get(column_name)
 
-    def coverage(self, cov_type: CoverageType):
-        coverages = [col.coverage(cov_type) for col in self.columns.values()]
-        covered = set(f'{self.name}.{col}' for covered, _ in coverages for col in covered)
-        total = set(f'{self.name}.{col}' for _, total in coverages for col in total)
-        return covered, total
-
 
 @dataclass
-class Catalog(EntityWithCoverage):
+class Catalog:
     """Dataclass containing the information about a database catalog, its tables and columns."""
 
     tables: Dict[str, Table]
 
     @staticmethod
     def from_nodes(nodes):
-        return Catalog({node['metadata']['name']: Table.from_node(node) for node in nodes})
+        tables = [Table.from_node(table) for table in nodes]
+        return Catalog({table.name: table for table in tables})
 
     def get_table(self, table_name):
         return self.tables.get(table_name)
 
-    def coverage(self, cov_type: CoverageType):
-        coverages = [table.coverage(cov_type) for table in self.tables.values()]
-        covered = set(col for covered, _ in coverages for col in covered)
-        total = set(col for _, total in coverages for col in total)
-        return covered, total
-
 
 @dataclass
 class Manifest:
     sources: Dict[str, Dict[str, Dict]]
     models: Dict[str, Dict[str, Dict]]
+    seeds: Dict[str, Dict[str, Dict]]
     tests: Dict[str, Dict[str, List[Dict]]]
 
     @classmethod
-    def from_nodes(cls, manifest_nodes: Dict[str: Dict]) -> Manifest:
-        id_to_table_name = {table_id: table['name'] for table_id, table in manifest_nodes.items()
-                            if table['resource_type'] in ['source', 'model']}
-        sources = {table['name']: table['columns'] for table in manifest_nodes.values()
-                   if table['resource_type'] == 'source'}
-        models = {table['name']: table['columns'] for table in manifest_nodes.values()
-                  if table['resource_type'] == 'model'}
+    def from_nodes(cls, manifest_nodes: Dict[str, Dict]) -> Manifest:
+        """Constructs a ``Manifest`` by parsing from manifest.json nodes."""
+
+        sources = [table for table in manifest_nodes.values()
+                   if table['resource_type'] == 'source']
+        sources = {cls._full_table_name(table): cls._normalize_column_names(table['columns'])
+                   for table in sources}
+
+        models = [table for table in manifest_nodes.values() if table['resource_type'] == 'model']
+        models = {cls._full_table_name(table): cls._normalize_column_names(table['columns'])
+                  for table in models}
+
+        seeds = [table for table in manifest_nodes.values() if table['resource_type'] == 'seed']
+        seeds = {cls._full_table_name(table): cls._normalize_column_names(table['columns'])
+                 for table in seeds}
+
+        tests = cls._parse_tests(manifest_nodes)
+
+        return Manifest(sources, models, seeds, tests)
+
+    @classmethod
+    def _parse_tests(cls, manifest_nodes: Dict[str, Dict]) -> Dict[str, Dict[str, List[Dict]]]:
+        """Parses tests from manifest.json nodes.
+
+        The logic is taken from the dbt-docs official source code:
+        https://github.com/dbt-labs/dbt-docs/blob/da84e95ab9febde3127ed28ff698516b2a610532/src/app/services/project_service.js#L149-L215
+        """
+
+        id_to_table_name = {table_id: cls._full_table_name(table)
+                            for table_id, table in manifest_nodes.items()
+                            if table['resource_type'] in ['source', 'model', 'seed']}
+
         tests = {}
         for node in manifest_nodes.values():
             if node['resource_type'] != 'test' or 'schema' not in node['tags']:
@@ -128,21 +127,33 @@ class Manifest:
                 continue
 
             if node['test_metadata']['name'] == 'relationships':
-                table_id = depends_on[len(depends_on) - 1]
+                table_id = depends_on[-1]
             else:
                 table_id = depends_on[0]
             table_name = id_to_table_name[table_id]
-            column_name = node['column_name'] or node['test_metadata']['kwargs']['column_name'] \
-                or node['test_metadata']['kwargs']['arg']
 
+            column_name = node.get('column_name') \
+                or node['test_metadata']['kwargs'].get('column_name') \
+                or node['test_metadata']['kwargs'].get('arg')
             if not column_name:
                 continue
 
+            column_name = column_name.lower()
             table_tests = tests.setdefault(table_name, {})
             column_tests = table_tests.setdefault(column_name, [])
             column_tests.append(node)
 
-        return Manifest(sources, models, tests)
+        return tests
+
+    @staticmethod
+    def _full_table_name(table):
+        return f'{table["schema"]}.{table["name"]}'.lower()
+
+    @staticmethod
+    def _normalize_column_names(columns):
+        for col in columns.values():
+            col['name'] = col['name'].lower()
+        return {col['name']: col for col in columns.values()}
 
 
 @dataclass
@@ -163,17 +174,22 @@ class CoverageReport:
             ``Catalog`` and for ``Column``s of ``Table``s.
     """
 
-    class Type(Enum):
+    class EntityType(Enum):
         CATALOG = 'catalog'
         TABLE = 'table'
         COLUMN = 'column'
 
-    report_type: Type
+    @dataclass(frozen=True)
+    class ColumnRef:
+        table_name: str
+        column_name: str
+
+    entity_type: EntityType
     cov_type: CoverageType
     entity_name: Optional[str]
-    covered: Set[str]
-    total: Set[str]
-    misses: Set[str] = field(init=False)
+    covered: Set[ColumnRef]
+    total: Set[ColumnRef]
+    misses: Set[ColumnRef] = field(init=False)
     coverage: float = field(init=False)
     subentities: Dict[str, CoverageReport]
 
@@ -187,46 +203,64 @@ class CoverageReport:
 
     @classmethod
     def from_catalog(cls, catalog: Catalog, cov_type: CoverageType):
-        cov = catalog.coverage(cov_type)
+        subentities = {table.name: CoverageReport.from_table(table, cov_type)
+                       for table in catalog.tables.values()}
+        covered = set(col for table_report in subentities.values() for col in table_report.covered)
+        total = set(col for table_report in subentities.values() for col in table_report.total)
+
         return CoverageReport(
-            cls.Type.CATALOG,
+            cls.EntityType.CATALOG,
             cov_type,
             None,
-            cov[0],
-            cov[1],
-            {table.name: CoverageReport.from_table(table, cov_type)
-             for table in catalog.tables.values()}
+            covered,
+            total,
+            subentities
         )
 
     @classmethod
     def from_table(cls, table: Table, cov_type: CoverageType):
-        cov = table.coverage(cov_type)
+        subentities = {col.name: CoverageReport.from_column(col, cov_type)
+                       for col in table.columns.values()}
+        covered = set(replace(col, table_name=table.name) for col_report in subentities.values()
+                      for col in col_report.covered)
+        total = set(replace(col, table_name=table.name) for col_report in subentities.values()
+                    for col in col_report.total)
+
         return CoverageReport(
-            cls.Type.TABLE,
+            cls.EntityType.TABLE,
             cov_type,
             table.name,
-            cov[0],
-            cov[1],
-            {col.name: CoverageReport.from_column(col, cov_type) for col in table.columns.values()}
+            covered,
+            total,
+            subentities
         )
 
     @classmethod
     def from_column(cls, column: Column, cov_type: CoverageType):
-        cov = column.coverage(cov_type)
+        if cov_type == CoverageType.DOC:
+            covered = column.doc
+        elif cov_type == CoverageType.TEST:
+            covered = column.test
+        else:
+            raise ValueError(f"Unsupported cov_type {cov_type}")
+
+        covered = {CoverageReport.ColumnRef(None, column.name)} if covered else set()
+        total = {CoverageReport.ColumnRef(None, column.name)}
+
         return CoverageReport(
-            cls.Type.COLUMN,
+            cls.EntityType.COLUMN,
             cov_type,
             column.name,
-            cov[0],
-            cov[1],
+            covered,
+            total,
             {}
         )
 
     def to_formatted_string(self):
-        if self.report_type == CoverageReport.Type.TABLE:
+        if self.entity_type == CoverageReport.EntityType.TABLE:
             return f"{self.entity_name:50} {len(self.covered):5}/{len(self.total):<5} " \
                    f"{self.coverage * 100:5.1f}%"
-        elif self.report_type == CoverageReport.Type.CATALOG:
+        elif self.entity_type == CoverageReport.EntityType.CATALOG:
             buf = io.StringIO()
 
             buf.write("Coverage report\n")
@@ -240,17 +274,17 @@ class CoverageReport:
             return buf.getvalue()
         else:
             raise TypeError(f"Unsupported report_type for to_formatted_string method: "
-                            f"{type(self.report_type)}")
+                            f"{type(self.entity_type)}")
 
     def to_dict(self):
-        if self.report_type == CoverageReport.Type.COLUMN:
+        if self.entity_type == CoverageReport.EntityType.COLUMN:
             return {
                 'name': self.entity_name,
                 'covered': len(self.covered),
                 'total': len(self.total),
                 'coverage': self.coverage
             }
-        elif self.report_type == CoverageReport.Type.TABLE:
+        elif self.entity_type == CoverageReport.EntityType.TABLE:
             return {
                 'name': self.entity_name,
                 'covered': len(self.covered),
@@ -258,7 +292,7 @@ class CoverageReport:
                 'coverage': self.coverage,
                 'columns': [col_report.to_dict() for col_report in self.subentities.values()]
             }
-        elif self.report_type == CoverageReport.Type.CATALOG:
+        elif self.entity_type == CoverageReport.EntityType.CATALOG:
             return {
                 'cov_type': self.cov_type,
                 'covered': len(self.covered),
@@ -268,7 +302,7 @@ class CoverageReport:
             }
         else:
             raise TypeError(f"Unsupported report_type for to_dict method: "
-                            f"{type(self.report_type)}")
+                            f"{type(self.entity_type)}")
 
     @staticmethod
     def from_dict(report, cov_type: CoverageType):
@@ -276,15 +310,11 @@ class CoverageReport:
             subentities = {table_report['name']: CoverageReport.from_dict(table_report, cov_type)
                            for table_report in report['tables']}
             return CoverageReport(
-                CoverageReport.Type.CATALOG,
+                CoverageReport.EntityType.CATALOG,
                 cov_type,
                 None,
-                set(f"{tbl.entity_name}.{col}"
-                    for tbl in subentities.values()
-                    for col in tbl.covered),
-                set(f"{tbl.entity_name}.{col}"
-                    for tbl in subentities.values()
-                    for col in tbl.total),
+                set(col for tbl in subentities.values() for col in tbl.covered),
+                set(col for tbl in subentities.values() for col in tbl.total),
                 subentities
             )
         elif 'columns' in report:
@@ -292,20 +322,23 @@ class CoverageReport:
             subentities = {col_report['name']: CoverageReport.from_dict(col_report, cov_type)
                            for col_report in report['columns']}
             return CoverageReport(
-                CoverageReport.Type.TABLE,
+                CoverageReport.EntityType.TABLE,
                 cov_type,
                 table_name,
-                set(col.entity_name for col in subentities.values() for _ in col.covered),
-                set(col.entity_name for col in subentities.values() for _ in col.total),
+                set(replace(col, table_name=table_name) for col_report in subentities.values()
+                    for col in col_report.covered),
+                set(replace(col, table_name=table_name) for col_report in subentities.values()
+                    for col in col_report.total),
                 subentities
             )
         else:
+            column_name = report['name']
             return CoverageReport(
-                CoverageReport.Type.COLUMN,
+                CoverageReport.EntityType.COLUMN,
                 cov_type,
-                report['name'],
-                {report['name']} if report['covered'] > 0 else set(),
-                {report['name']},
+                column_name,
+                {CoverageReport.ColumnRef(None, column_name)} if report['covered'] > 0 else set(),
+                {CoverageReport.ColumnRef(None, column_name)},
                 {}
             )
 
@@ -325,35 +358,36 @@ class CoverageDiff:
         assert self.before is None or self.before.cov_type == self.after.cov_type, \
             f"Cannot compare reports with different cov_types: {self.before.cov_type} and " \
             f"{self.after.cov_type}"
-        assert self.before is None or self.before.report_type == self.after.report_type, \
+        assert self.before is None or self.before.entity_type == self.after.entity_type, \
             f"Cannot compare reports with different report_types: {self.before.report_type} and " \
-            f"{self.after.report_type}"
+            f"{self.after.entity_type}"
 
         self.new_misses = self.find_new_misses()
 
     def find_new_misses(self):
-        if self.after.report_type == CoverageReport.Type.COLUMN:
+        if self.after.entity_type == CoverageReport.EntityType.COLUMN:
             return None
 
-        new_misses_names = self.after.misses - (self.before.misses if self.before is not None
-                                                else set())
-        new_misses_entity_names = set(miss.split('.', maxsplit=1)[0] for miss in new_misses_names)
+        new_misses = self.after.misses - (self.before.misses if self.before is not None else set())
 
         res: Dict[str, CoverageDiff] = {}
-        for new_miss_entity_name in new_misses_entity_names:
-            before_entity = self.before.subentities.get(new_miss_entity_name) \
+        for new_miss in new_misses:
+            new_misses_entity_name = new_miss.table_name \
+                if self.after.entity_type == CoverageReport.EntityType.CATALOG \
+                else new_miss.column_name
+            before_entity = self.before.subentities.get(new_misses_entity_name) \
                 if self.before is not None else None
-            after_entity = self.after.subentities[new_miss_entity_name]
-            res[new_miss_entity_name] = CoverageDiff(before_entity, after_entity)
+            after_entity = self.after.subentities[new_misses_entity_name]
+            res[new_misses_entity_name] = CoverageDiff(before_entity, after_entity)
 
         return res
 
     def summary(self):
         buf = io.StringIO()
 
-        if self.after.report_type != CoverageReport.Type.CATALOG:
+        if self.after.entity_type != CoverageReport.EntityType.CATALOG:
             raise TypeError(f"Unsupported report_type for summary method: "
-                            f"{self.after.report_type}")
+                            f"{self.after.entity_type}")
 
         buf.write(f"{'':10}{'before':>10}{'after':>10}{'+/-':>15}\n")
         buf.write('=' * 45 + "\n")
@@ -388,10 +422,10 @@ class CoverageDiff:
         return buf.getvalue()
 
     def new_misses_summary(self):
-        if self.after.report_type == CoverageReport.Type.COLUMN:
+        if self.after.entity_type == CoverageReport.EntityType.COLUMN:
             return self._new_miss_summary_row()
 
-        elif self.after.report_type == CoverageReport.Type.TABLE:
+        elif self.after.entity_type == CoverageReport.EntityType.TABLE:
             buf = io.StringIO()
 
             buf.write(self._new_miss_summary_row())
@@ -400,7 +434,7 @@ class CoverageDiff:
 
             return buf.getvalue()
 
-        elif self.after.report_type == CoverageReport.Type.CATALOG:
+        elif self.after.entity_type == CoverageReport.EntityType.CATALOG:
             buf = io.StringIO()
             buf.write("=" * 94 + '\n')
             buf.write(self._new_miss_summary_row())
@@ -413,20 +447,20 @@ class CoverageDiff:
 
         else:
             raise TypeError(f"Unsupported report_type for new_misses_summary method: "
-                            f"{self.after.report_type}")
+                            f"{self.after.entity_type}")
 
     def _new_miss_summary_row(self):
-        if self.after.report_type == CoverageReport.Type.CATALOG:
+        if self.after.entity_type == CoverageReport.EntityType.CATALOG:
             title_prefix = ''
-        elif self.after.report_type == CoverageReport.Type.TABLE:
+        elif self.after.entity_type == CoverageReport.EntityType.TABLE:
             title_prefix = '- '
-        elif self.after.report_type == CoverageReport.Type.COLUMN:
+        elif self.after.entity_type == CoverageReport.EntityType.COLUMN:
             title_prefix = '-- '
         else:
             raise TypeError(f"Unsupported report_type for _new_miss_summary_row method: "
-                            f"{type(self.after.report_type)}")
+                            f"{type(self.after.entity_type)}")
 
-        title = "Catalog" if self.after.report_type == CoverageReport.Type.CATALOG \
+        title = "Catalog" if self.after.entity_type == CoverageReport.EntityType.CATALOG \
             else self.after.entity_name
         title = title_prefix + title
 
@@ -453,7 +487,7 @@ def load_catalog(project_dir: Path) -> Catalog:
     catalog_nodes = {**catalog_json['sources'], **catalog_json['nodes']}
     catalog = Catalog.from_nodes(catalog_nodes.values())
 
-    logging.info("Successfully loaded %d catalog nodes", len(catalog_nodes))
+    logging.info("Successfully loaded %d tables from catalog", len(catalog.tables))
 
     return catalog
 
@@ -478,14 +512,18 @@ def load_files(project_dir: Path) -> Catalog:
         catalog_table = catalog.get_table(table_name)
         manifest_source_table = manifest.sources.get(table_name, {})
         manifest_model_table = manifest.models.get(table_name, {})
+        manifest_seed_table = manifest.seeds.get(table_name, {})
         manifest_table_tests = manifest.tests.get(table_name, {})
 
         for catalog_column in catalog_table.columns.values():
             manifest_source_column = manifest_source_table.get(catalog_column.name)
             manifest_model_column = manifest_model_table.get(catalog_column.name)
+            manifest_seed_column = manifest_seed_table.get(catalog_column.name)
             manifest_column_tests = manifest_table_tests.get(catalog_column.name)
 
-            doc = (manifest_source_column or manifest_model_column or {}).get('description')
+            manifest_column = manifest_source_column or manifest_model_column \
+                or manifest_seed_column or {}
+            doc = manifest_column.get('description')
             catalog_column.doc = Column.is_valid_doc(doc)
             catalog_column.test = Column.is_valid_test(manifest_column_tests)
 
